"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KycClient = void 0;
const __1 = require("..");
class KycClient {
    constructor(ctx, pda) {
        this.ctx = ctx;
        this.pda = pda;
    }
    static getClient(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(ctx.program.programId);
            return new KycClient(ctx, pda);
        });
    }
    createProviderConfig(provider, name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!name || name === "") {
                throw Error("Provider name is required");
            }
            if (name.length > 20) {
                throw Error("Invalid Provider name length, maximum 20 characters");
            }
            const providerConfig = this.pda.provider_config(name);
            const tx = (yield this.ctx.methods.createProviderConfig({
                accounts: {
                    authority: this.ctx.wallet.publicKey,
                    providerConfigAccount: providerConfig.key,
                },
                inputs: {
                    name,
                    provider,
                },
            })).toTx();
            return tx;
        });
    }
    deactivateProvider(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const providerConfig = this.pda.provider_config(name);
            const providerConfigData = yield this.getProviderConfigByProviderName(name);
            if (providerConfigData.admin.toBase58() !== this.ctx.wallet.publicKey.toBase58()) {
                throw Error("Only Admin can deactivate provider");
            }
            const tx = (yield this.ctx.methods.deactivateProvider({
                accounts: {
                    authority: this.ctx.wallet.publicKey,
                    providerConfigAccount: providerConfig.key,
                },
                inputs: {},
            })).toTx();
            return tx;
        });
    }
    deactivateUser(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const userConfig = this.pda.user_config(user);
            const userConfigData = yield this.getOneUserConfig(userConfig.key);
            if (userConfigData.admin.toBase58() !== this.ctx.wallet.publicKey.toBase58()) {
                throw Error("Only Admin can deactivate user");
            }
            const tx = (yield this.ctx.methods.deactivateUser({
                accounts: {
                    authority: this.ctx.wallet.publicKey,
                    userConfig: userConfig.key,
                },
                inputs: {},
            })).toTx();
            return tx;
        });
    }
    getProviderConfigByProviderName(providerName) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(this.ctx.program.programId);
            const providerConfig = pda.provider_config(providerName);
            const providerConfigData = yield this.ctx.fetcher.getProviderConfig(providerConfig.key, true);
            if (!providerConfigData) {
                throw new Error(`Provider Config of provider name=${providerName} not found`);
            }
            return providerConfigData;
        });
    }
    getUserConfig(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(this.ctx.program.programId);
            const userConfig = pda.user_config(user);
            const userConfigData = yield this.ctx.fetcher.getUserConfig(userConfig.key, true);
            if (!userConfigData) {
                throw new Error(`User Config of user ${user} not found`);
            }
            return userConfigData;
        });
    }
    getOneUserConfig(userConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const userConfigData = yield this.ctx.fetcher.getUserConfig(userConfig, true);
            if (!userConfigData) {
                throw new Error(`User Config ${userConfig} not found`);
            }
            return userConfigData;
        });
    }
    getCurrentUserKyc(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(this.ctx.program.programId);
            const userConfig = pda.user_config(user);
            const userConfigData = yield this.ctx.fetcher.getUserConfig(userConfig.key, true);
            if (!userConfigData) {
                throw new Error(`User Config of user ${user} not found`);
            }
            return this.getOneUserKyc(userConfigData.latestKycAccount);
        });
    }
    getOneUserKyc(userKyc) {
        return __awaiter(this, void 0, void 0, function* () {
            const userKycData = yield this.ctx.fetcher.getOneUserKyc(userKyc, true);
            if (!userKycData) {
                throw new Error(`User Kyc ${userKyc} not found`);
            }
            const userConfigData = yield this.getOneUserConfig(userKycData.userConfigAccount);
            if (userConfigData.provider.toBase58() === __1.RENEC_PROVIDER_TESTNET) {
                userKycData.provider = "RENEC blockchain";
            }
            else {
                userKycData.provider = userConfigData.provider.toBase58();
            }
            return userKycData;
        });
    }
    isKyc(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const pda = new __1.PDA(this.ctx.program.programId);
            const userConfig = pda.user_config(user);
            const userConfigData = yield this.ctx.fetcher.getUserConfig(userConfig.key, true);
            if (!userConfigData) {
                return {
                    isKyc: false,
                    kycLevel: null,
                    isExpired: null,
                };
            }
            const userKycData = yield this.getOneUserKyc(userConfigData.latestKycAccount);
            return {
                isKyc: true,
                kycLevel: userConfigData.kycLevel,
                isExpired: userKycData.isExpired,
            };
        });
    }
}
exports.KycClient = KycClient;
