"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateAndHashKycParams = exports.stringToBytes32 = exports.keypairFromJson = void 0;
const anchor = __importStar(require("@project-serum/anchor"));
const types_1 = require("../types");
const countryLookup = __importStar(require("country-code-lookup"));
const { createHash } = require("crypto");
function keypairFromJson(secretKey) {
    return anchor.web3.Keypair.fromSecretKey(Uint8Array.from(secretKey));
}
exports.keypairFromJson = keypairFromJson;
function hashNTimes(string, salt, n) {
    if (!string || string === "")
        return null;
    let hash = createHash("sha256");
    let result = hash.update(string).update(salt).digest("hex");
    for (let i = 1; i < n; i++) {
        let hashInterval = createHash("sha256");
        hashInterval.update(result);
        result = hashInterval.digest("hex");
    }
    return result;
}
const stringToBytes32 = (str) => {
    if (!str)
        return [];
    const encoder = new TextEncoder();
    const encodedString = encoder.encode(str);
    const bytes32 = new Uint8Array(32);
    bytes32.set(encodedString.subarray(0, 32));
    const numbersArray = Array.from(bytes32, (byte) => byte);
    return numbersArray;
};
exports.stringToBytes32 = stringToBytes32;
const validateAndHashKycParams = (client, salt, name, documentId, country, dob, doe, gender) => {
    if (!salt || salt === "") {
        throw Error("Salt is required!");
    }
    if (salt.length > 20) {
        throw Error("Invalid salt length! Cannot exceed 20 characters");
    }
    if (name.length > 255 || documentId.length > 255) {
        throw Error("Invalid parameter length! Cannot exceed 255 characters");
    }
    if (country.length !== 2) {
        throw Error("Invalid country! Please provider valid country code");
    }
    let encodedCountry = country.toLowerCase();
    if (!countryLookup.byFips(encodedCountry)) {
        throw Error("Invalid country");
    }
    let encodedGender = [];
    if (gender) {
        if (!Object.values(types_1.Gender).includes(gender)) {
            throw Error("Invalid gender");
        }
        encodedGender = (0, exports.stringToBytes32)(hashNTimes(gender, salt, client.genderHashRound));
    }
    return {
        encodedName: (0, exports.stringToBytes32)(hashNTimes(name, salt, client.nameHashRound)),
        encodedDocumentId: (0, exports.stringToBytes32)(hashNTimes(documentId, salt, client.documentIdHashRound)),
        encodedCountry,
        encodedDob: (0, exports.stringToBytes32)(hashNTimes(dob.getTime().toString(), salt, client.dobHashRound)),
        encodedDoe: doe
            ? (0, exports.stringToBytes32)(hashNTimes(doe.getTime().toString(), salt, client.doeHashRound))
            : [],
        encodedGender,
    };
};
exports.validateAndHashKycParams = validateAndHashKycParams;
